import Flutter
import UIKit
import ReplayKit

public class SwiftVideosdkPlugin: NSObject, FlutterPlugin {

  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "videosdk", binaryMessenger: registrar.messenger())
    let instance = SwiftVideosdkPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)

    let eventChannel = FlutterEventChannel(name: "videosdk-event", binaryMessenger: registrar.messenger())
    let streamHandler = ScreenShareEventStreamHandler();
    eventChannel.setStreamHandler(streamHandler)    
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
      if(call.method == "getPlatformVersion"){
          result("iOS " + UIDevice.current.systemVersion)
      return
          
      }else if(call.method == "getDeviceInfo") {
              var deviceInfo = [String: String]()
              deviceInfo["brand"] = "Apple"
          deviceInfo["model"] = UIDevice.modelName
              deviceInfo["osVersion"] = UIDevice.current.systemVersion
              result(deviceInfo)
          return
          }
          else if(call.method == "requestScreenSharePermission") {
              DispatchQueue.main.async { [self] in
            let pickerView = RPSystemBroadcastPickerView(
                frame: CGRect(x: 0, y: 0, width: 0, height: 0))
            var tap = pickerView.subviews.first as! UIButton
            pickerView.translatesAutoresizingMaskIntoConstraints = false
            let extensionId = Bundle.main.object(forInfoDictionaryKey: "RTCScreenSharingExtension") as? String
            pickerView.preferredExtension = extensionId
            tap.sendActions(for: .touchUpInside)
            
            }
          return
          }
  }
}

public class ScreenShareEventStreamHandler: NSObject, FlutterStreamHandler {
    var _eventSink: FlutterEventSink?

    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        _eventSink = events
        let notificationCenter = CFNotificationCenterGetDarwinNotifyCenter()
        let observer = UnsafeRawPointer(Unmanaged.passUnretained(self).toOpaque())
        let notificationStartName = "videosdk.flutter.startScreenShare"as CFString
        let notificationStopName = "videosdk.flutter.stopScreenShare"as CFString

        CFNotificationCenterAddObserver(notificationCenter,
                                        observer,
                                        { (_, observer, name, _, _) -> Void in
                                            let mySelf = Unmanaged<ScreenShareEventStreamHandler>.fromOpaque(
                    observer!).takeUnretainedValue()
                                            mySelf.sendEvent(event:"STOP_SCREENSHARE")
                                        },
                                        notificationStopName,
                                        nil,
                                        CFNotificationSuspensionBehavior.deliverImmediately)
        
        CFNotificationCenterAddObserver(notificationCenter,
                                        observer,
                                        { (_, observer, name, _, _) -> Void in
                                            let mySelf = Unmanaged<ScreenShareEventStreamHandler>.fromOpaque(
                    observer!).takeUnretainedValue()
                                            mySelf.sendEvent(event:"START_SCREENSHARE")
                                        },
                                        notificationStartName,
                                        nil,
                                        CFNotificationSuspensionBehavior.deliverImmediately)
        return nil
    }

    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        _eventSink = nil
        return nil
    }
    // This method is called by the speech controller as mentioned above
    @objc func sendEvent(event: String) {
        _eventSink?(event)
    }
}

public extension UIDevice {

    static let modelName: String = {
        var systemInfo = utsname()
        uname(&systemInfo)
        let machineMirror = Mirror(reflecting: systemInfo.machine)
        let identifier = machineMirror.children.reduce("") { identifier, element in
            guard let value = element.value as? Int8, value != 0 else { return identifier }
            return identifier + String(UnicodeScalar(UInt8(value)))
        }

        return identifier
    }()

}
